//  ReadWriteController.m

#import "ReadWriteController.h"
#import "PhyUtility.h"
#import "ModelSettings.h"
#import "GlobalDefine.h"
#import "Utils.h"
#import <AsReaderDockSDK/AsReaderDockSDK.h>
#import "ErrorCodeHelper.h"

@interface ReadWriteController ()<UITextFieldDelegate, AsReaderRFIDDeviceDelegate, AsReaderDeviceDelegate>

@property (weak, nonatomic) IBOutlet UISegmentedControl *olMode;
@property (weak, nonatomic) IBOutlet UITextField *olAccessPassword;
@property (weak, nonatomic) IBOutlet UITextField *olStartAdress;
@property (weak, nonatomic) IBOutlet UITextField *olLength;
@property (weak, nonatomic) IBOutlet UITextField *olData;
@property (weak, nonatomic) IBOutlet UISegmentedControl *olTargetMemory;
@property (weak, nonatomic) IBOutlet UITextField *olTargetEpc;
@property (weak, nonatomic) IBOutlet UILabel *olDataTitle;
@property (weak, nonatomic) IBOutlet UIScrollView *olScrollVeiw;
@end

@implementation ReadWriteController
{
    CGFloat y;
    CGFloat height;
}
#pragma mark - Lifecycle
//----------------------------------------------------------------
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    self.olScrollVeiw.scrollEnabled = YES;
    self.olScrollVeiw.contentSize = CGSizeMake(self.view.frame.size.width, self.view.frame.size.height + 600);
    y = self.view.frame.origin.y;
    height = self.view.frame.size.height;
    [[NSNotificationCenter defaultCenter]addObserver:self selector:@selector(keyboardWillShow:) name:UIKeyboardWillShowNotification object:self.view.window];
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];

    self.olAccessPassword.delegate = self;
    self.olStartAdress.delegate = self;
    self.olLength.delegate = self;
    self.olData.delegate = self;
    self.olTargetEpc.delegate = self;
    
    ModelSettings *model = [ModelSettings sharedInstance];
    NSDictionary *dic = [model.m_arReadTagData objectAtIndex:model.m_nSelectTagNum];
    self.olTargetEpc.text = [PhyUtility getEpcStringFromPcEPCData:[dic objectForKey:df_CELL_TAG_RAW]];
    
    AsReaderDevice *device = [AsReaderRFIDDevice sharedInstance];
    device.delegateDevice = self;
    device.delegateRFID = self;
}

- (void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:animated];
    [self setOlAccessPassword:nil];
    [self setOlMode:nil];
    [self setOlStartAdress:nil];
    [self setOlLength:nil];
    [self setOlTargetMemory:nil];
    [self setOlData:nil];
    [self setOlTargetEpc:nil];
    [self setOlScrollVeiw:nil];
    AsReaderDevice *device = [AsReaderRFIDDevice sharedInstance];
    device.delegateDevice = nil;
    device.delegateRFID = nil;
}

#pragma mark - Private method
//----------------------------------------------------------------
- (void)setViewMoveUp:(BOOL)moveUp
{
    if(moveUp) {
        NSLog(@"Scroll MOVE UP");
        [UIView animateWithDuration:0.2 animations:^{self.olScrollVeiw.contentOffset = CGPointMake(0.0f, 100.f);}];
    } else {
        NSLog(@"Scroll MOVE Down");
        [UIView animateWithDuration:0.2 animations:^{self.olScrollVeiw.contentOffset = CGPointMake(0.0f, self.olScrollVeiw.frame.origin.y-80);}];
    }
}

- (void)keyboardWillShow:(NSNotification *)notif
{
    //olData
    if([self.olData isFirstResponder] &&self.view.frame.origin.y >=0) {
        [self setViewMoveUp:YES];
    } else if ([self.olData isFirstResponder]&& self.view.frame.origin.y <0) {
        [self setViewMoveUp:NO];
    }
}

- (NSString *)dataToEpcInteger:(NSData *)data
{
    uint8_t* pData = (uint8_t*)[data bytes];
    uint16_t intData = pData[0] << 8 | pData[1];
    intData = intData >>11;
    return [NSString stringWithFormat:@"%d",intData];
    
}
- (void)touchesBegan:(NSSet<UITouch *> *)touches withEvent:(UIEvent *)event
{
    [self.view endEditing:YES];
}
#pragma mark - UITextField delegate
//----------------------------------------------------------------
- (BOOL)textField:(UITextField *)textField shouldChangeCharactersInRange:(NSRange)range replacementString:(NSString *)string
{
    if (textField == self.olData) {
        if ([Utils isCorrectWriteData:[textField.text stringByAppendingString:string]]) {
            return YES;
        } else {
            return NO;
        }
    }
    return YES;
}

- (BOOL)textFieldShouldReturn:(UITextField *)txtField
{
    [self setViewMoveUp:NO];
    [txtField resignFirstResponder];
    return NO;
}
- (BOOL)textFieldShouldBeginEditing:(UITextField *)textField
{
    if (textField == _olData) {
        [UIView animateWithDuration:0.1 animations:^{
            self.view.frame = CGRectMake(self.view.frame.origin.x, self->y - 100, self.view.frame.size.width, self->height);
        }];
    }
    return YES;
}
- (BOOL)textFieldShouldEndEditing:(UITextField *)textField
{
    if (textField == _olData) {
        [UIView animateWithDuration:0.1 animations:^{
            self.view.frame = CGRectMake(self.view.frame.origin.x, self->y + 64, self.view.frame.size.width, self->height);
        }];
    }
    return YES;
}
#pragma mark - AsReaderRFID delegate
//----------------------------------------------------------------
- (void)tagMemoryReceived:(NSData *)data
{
    dispatch_async(dispatch_get_main_queue(), ^{
       NSString *strData;
       if (self.olTargetMemory.selectedSegmentIndex == 1) {
           strData = [self dataToEpcInteger:data];
       } else {
           strData = [data bytesToHexString];
       }
       self.olData.text = strData;
   });
}

- (void)writtenReceived:(int)statusCode
{
    if(statusCode == 0x00) {
        dispatch_async(dispatch_get_main_queue(), ^{
            [Utils showAlert:@"Confirm" message:@"Done Write." view:self];
       });
    }
}

- (void)errorReceived:(NSData *)errorCode
{
    ErrorCodeHelper *errorHelper = [ErrorCodeHelper sharedInstance];
      NSString *errorResult = [errorHelper errorReceived:errorCode];
      
    dispatch_async(dispatch_get_main_queue(), ^{
              [Utils showAlert:@"" message:errorResult view:self];
          });
}

#pragma mark - IBAction
//----------------------------------------------------------------
- (IBAction)eModeChanged:(id)sender
{
    if(self.olMode.selectedSegmentIndex == 0) {
        self.navigationItem.title = @"Read";
    } else {
        self.navigationItem.title = @"Write";
    }
}


- (IBAction)rightBarButtonItemClicked:(id)sender
{
    [self.olTargetEpc resignFirstResponder];
    [self.olAccessPassword resignFirstResponder];
    [self.olStartAdress resignFirstResponder];
    [self.olLength resignFirstResponder];
    [self.olData resignFirstResponder];
    [self setViewMoveUp:NO];
    NSString *hexaString = self.olAccessPassword.text;
    NSScanner *pScanner = [NSScanner scannerWithString: hexaString];
    uint32_t accesspassword;
    [pScanner scanHexInt: &accesspassword];
    
    uint16_t startAddress = [self.olStartAdress.text intValue];
    uint16_t dataLength = [self.olLength.text intValue];
    uint8_t membank = '\0';
    
    
    switch(self.olTargetMemory.selectedSegmentIndex){
        case 0:
            membank = 0;
            break;
            
        case 1: //PC
        case 2: //EPC
            membank = 1;
            break;
            
        case 3: // TID
            membank = 2;
            break;
            
        case 4:
            membank = 3;
            break;
    }
    
    NSData *data = [self.olData.text hexStringToBytes];
    NSData *epcs = [self.olTargetEpc.text hexStringToBytes];
    if(epcs.length == 0 || epcs.length < 2) {
        return;
    }
    
    NSData *pcData = [epcs subdataWithRange:NSMakeRange(0, 2)];
    
    if(self.olMode.selectedSegmentIndex == 0) {
        [[AsReaderRFIDDevice sharedInstance] readTagWithAccessPassword:accesspassword epc:epcs memoryBank:membank startAddress:startAddress dataLength:dataLength];
    
    } else {
        if (self.olTargetMemory.selectedSegmentIndex == 1) {
            uint8_t *pData = (uint8_t *)[pcData bytes];
            uint16_t intData = pData[0] << 8 | pData[1];
            int epcLength = [self.olData.text intValue];
            if (epcLength != 0) {
                intData = intData & 0x07FF;
                intData |= (epcLength & 0x1F) << 11;
            }
            data = [[NSString stringWithFormat:@"%04X",intData]hexStringToBytes];
        }
        [[AsReaderRFIDDevice sharedInstance] writeTagMemoryWithAccessPassword:accesspassword epc:epcs memoryBank:membank startAddress:startAddress dataToWrite:data];
    }
}

- (IBAction)eTargetMemoryChanged:(id)sender
{
    self.olDataTitle.text = @"Data (HEX)";
    [self.olData resignFirstResponder];
    [self.olData setKeyboardType:UIKeyboardTypeDefault];
    
    if (self.olTargetMemory.selectedSegmentIndex == 1) {
        //PC
        self.olStartAdress.text = @"1";
        self.olLength.text = @"1";
        self.olStartAdress.enabled = NO;
        self.olLength.enabled = NO;
        self.olDataTitle.text = @"EPC Length";
        [self.olData setKeyboardType:UIKeyboardTypeNumberPad];
    
    } else if(self.olTargetMemory.selectedSegmentIndex == 2) {
        //EPC
        self.olStartAdress.text = @"2";
        self.olLength.text = @"0";
        self.olStartAdress.enabled = NO;
        self.olLength.enabled = NO;
    
    } else {
        self.olStartAdress.enabled = YES;
        self.olLength.enabled = YES;
    }
}

- (IBAction)actLimitEPCLength:(id)sender
{
    if (self.olTargetMemory.selectedSegmentIndex == 1) {
        // 31까지 숫자 제한 : 5bit
        if ([self.olData.text intValue] > 31) {
            self.olData.text = @"31";
        }
    }
}

- (void)plugged:(BOOL)plug {
    
}

- (void)readerConnected:(int)status {
    
}
@end



